(*
Filename: Semantics.v

Formalization of the system actions pre and post-conditions.

*)
Require Export State.

Require Export Actions.

Section SemInstall.

(*Predicate that holds when component is not in the system*)
Definition cmpNotInState (c:Cmp) (s:State) : Prop :=
   forall (c':Cmp) (a:App),
    (apps s) a -> 
     (cmp (manifest a)) c' ->
      getCmpId c <> getCmpId c'.

(*Device manufacturer certificate*)
Parameter manufactCert : Cert.

(*Predicate that holds when application belongs to the device manufacturer. *)
Definition isManufacturerApp (a:App) : Prop :=
   cert a = manufactCert.

(*Predicate that holds when the user authorizes granting a permission *)
Parameter usrAuth : Perm -> Prop.

(*Predicate to check if a permission was defined by the user in a given state*)
Definition usrDefPerm (p:Perm)(s:State) : Prop :=
   exists a:App, (apps s) a /\
    (defPerms s) a p.

(*Grant permissions to the application *)
Definition authPerms (a:App)(s:State) : Prop :=
   forall p:Perm,
    use (manifest a) p ->
     isSystemPerm p \/ usrDefPerm p s -> (*If the permission does not
                                         exist, or is defined by the
                                         application, we ignore it*)
      let plevel := pl p in
        match plevel with
        | normal => True
        | dangerous => usrAuth p
        | signature => (~isSystemPerm p -> 
                         exists a':App, (apps s) a' /\
                         let mfst := manifest a' in
                           (usrP mfst) p /\ (cert a') = (cert a) ) /\
                       isSystemPerm p -> isManufacturerApp a
        | signatureOrSys => exists a':App, (systemImage s) a' /\
                             (cert a') = (cert a)
        end.

(*Install pre-condition*)
Definition pre_install (a:App) (s:State) : Prop := 
   (forall a':App, (apps s) a' -> idAp a <> idAp a') /\
   (forall (c1 c2:Cmp), cmp (manifest a) c1 ->
                          cmp (manifest a) c2 -> 
                           getCmpId c1 <> getCmpId c2) /\
   (forall c:Cmp, (cmp (manifest a)) c -> cmpNotInState c s) /\
   authPerms a s.

(*Initial value for application resources*)
Parameter initVal : Val.

(*Predicate to check whether a permission is defined in an application*)
Definition permDefInApp (p:Perm)(a:App) : Prop :=
   let mfst := manifest a in
     (usrP mfst) p.

(*Install post-condition*)
Definition post_install (a:App) (s s':State) : Prop := 
   (*addApp: Add application to system state*)
   (forall a':App, (apps s) a' -> (apps s') a') /\
   (forall a':App, (apps s') a' -> (apps s) a' \/ a' = a) /\
   (apps s') a /\
   (*addRes: Add application resources to system state*)
   (forall (a':App)(r:res)(v:Val), (resCont s) a' r v -> (resCont s') a' r v) /\
   (forall (a':App)(r:res)(v:Val), (resCont s') a' r v 
     -> (resCont s) a' r v \/ (a' = a /\ (appRes a) r /\ v = initVal)) /\
   (forall r:res, (appRes a) r -> (resCont s') a r initVal) /\
   (*addDefPerms: Add new user defined permissions to the system*)
   (forall (a':App)(p:Perm), (defPerms s) a' p -> (defPerms s') a' p) /\
   (forall (a':App)(p:Perm), (defPerms s') a' p -> (defPerms s) a' p \/
                                                    (a'=a /\ permDefInApp p a /\
                                                      ~isSystemPerm p /\
                                                       (forall (a'':App)(p':Perm), 
                                                        (apps s) a''
                                                         -> (defPerms s) a'' p'
                                                          -> idP p <> idP p'))) /\
   (forall (p:Perm), permDefInApp p a /\ ~ isSystemPerm p ->
     (forall (a'':App)(p':Perm), 
       (apps s) a''
        -> (defPerms s) a'' p'
         -> idP p <> idP p') ->
      (defPerms s') a p) /\
   (*grantPerms: Add permissions granted to application*)
   (forall (a':App)(p:Perm), (perms s) a' p -> (perms s') a' p) /\
   (forall (a':App)(p:Perm), (perms s') a' p -> (perms s) a' p \/
                                                 (a=a'/\(use (manifest a)) p/\
                                                  (isSystemPerm p\/usrDefPerm p s\/permDefInApp p a))) /\
   (forall p:Perm,
    (use (manifest a)) p ->
     isSystemPerm p \/ usrDefPerm p s \/ permDefInApp p a ->
      (perms s') a p) /\

   running s = running s' /\ delPPerms s = delPPerms s' /\ delTPerms s = delTPerms s' /\ systemImage s = systemImage s'.

End SemInstall.

Section SemUninstall.

(*Uninstall pre-condition*)
Definition pre_uninstall (a:App)(s:State) : Prop :=
    (apps s) a /\
    forall (ic:iCmp)(c:Cmp), (running s) ic c -> ~inApp c a.

(*Uninstall post-condition*)
Definition post_uninstall (a:App)(s s':State) : Prop := 
    (*removeApp: Remove the application from the state*)
    (forall a':App, (apps s') a' -> (apps s) a') /\
    (forall a':App, (apps s) a' -> (apps s') a' \/ a' = a) /\
    ~(apps s') a /\
    (*revokePerms: Revoke permissions granted to the application*)
    (forall (a':App)(p:Perm), (perms s') a' p -> (perms s) a' p) /\
    (forall (a':App)(p:Perm), (perms s) a' p -> (perms s') a' p \/ a' = a) /\
    (forall p:Perm, ~(perms s') a p) /\
    (*removeDefPerms: Remove permissions defined by the application*)
    (forall (a':App)(p:Perm), (defPerms s') a' p -> (defPerms s) a' p) /\
    (forall (a':App)(p:Perm), (defPerms s) a' p -> (defPerms s') a' p \/ a' = a) /\
    (forall p:Perm, ~(defPerms s') a p) /\
    (*removeRes: Remove the application resources*)
    (forall (a':App)(r:res)(v:Val), (resCont s') a' r v -> (resCont s) a' r v) /\
    (forall (a':App)(r:res)(v:Val), (resCont s) a' r v -> (resCont s') a' r v \/ a' = a) /\
    (forall (r:res)(v:Val), ~ (resCont s') a r v) /\
    (*revokeOtherTPerms: Remove temporarily delegated permissions to
      other running instances on resources of content providers of
      this application*)
    (forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType),
        (delTPerms s') ic cp u pt -> (delTPerms s) ic cp u pt) /\
    (forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType),
        (delTPerms s) ic cp u pt -> (delTPerms s') ic cp u pt \/ (cmp (manifest a)) (cmpCP cp)) /\
    (forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType),
     (cmp (manifest a)) (cmpCP cp) -> ~ (delTPerms s') ic cp u pt) /\
    (*- revokeSelfPPerms: Remove permissions delegated permanently to
      this application*)
    (* - revokeOtherPPerms: Remove permissions delegated permanently
      to other applications on resources of a content provider of this
      application*)
    (forall (a':App)(cp:CProvider)(u:uri)(pt:PType),
        (delPPerms s') a' cp u pt -> (delPPerms s) a' cp u pt) /\
    (forall (a':App)(cp:CProvider)(u:uri)(pt:PType),
        (delPPerms s) a' cp u pt -> (delPPerms s') a' cp u pt \/ a' = a \/ 
                                      (cmp (manifest a)) (cmpCP cp)) /\
    (forall (cp:CProvider)(u:uri)(pt:PType), ~ (delPPerms s') a cp u pt) /\
    (forall (a':App)(cp:CProvider)(u:uri)(pt:PType),
     (cmp (manifest a)) (cmpCP cp) -> ~ (delPPerms s') a' cp u pt) /\
    
    running s = running s' /\ systemImage s = systemImage s'.

End SemUninstall.

Section SemStart.

(*Component c1 has the permissions necessary to start component c2,
  sending an explicit intent*)
Definition canStart (c1 c2: Cmp)(s:State) : Prop :=
    forall (a1 a2:App)(H1:inApp c1 a1)(H2: inApp c2 a2),
     (apps s) a1 ->
      (apps s) a2 ->
        let m2 := manifest a2 in
         (exp m2) c2 H2 /\
         (forall (p:Perm), (cmpE m2) c2 H2 = Some p \/ 
           (cmpE m2) c2 H2 = None /\ appE m2 = Some p ->  
             (perms s) a1 p) \/(*a1 tiene los permisos de acceso y c2 está exportado*)
        a1 = a2 (*Si ambos componentes pertenecen a la misma app, los permisos no son necesarios*).

Definition insNotInState (ic:iCmp) (s:State) : Prop :=
    forall (ic':iCmp)(c:Cmp), (running s) ic' c ->  idICmp ic <> idICmp ic'.

(*Start pre-condition*)
Definition pre_start (ic:iCmp) (c:Cmp) (s:State) : Prop :=
    cmpInstalled c s /\ ~isCProvider c /\
    exists c':Cmp, (running s) ic c' /\ cmpInstalled c' s /\ ~isCProvider c' /\ canStart c' c s.

(*Start post-condition*)
Definition post_start (ic:iCmp) (c:Cmp) (s s':State) : Prop :=
    ( exists ic:iCmp, insNotInState ic s /\
       (*runCmp: Execution of ic in s'*)
       (forall (ic':iCmp)(c':Cmp), (running s) ic' c' -> (running s') ic' c') /\ 
       (forall (ic':iCmp)(c':Cmp), (running s') ic' c' -> (running s) ic' c' \/ (ic=ic'/\c=c')) /\
       (running s') ic c ) /\
    
    apps s = apps s' /\ perms s = perms s' /\ defPerms s = defPerms s' /\ delPPerms s = delPPerms s' /\ 
      delTPerms s = delTPerms s' /\ resCont s = resCont s' /\ systemImage s = systemImage s'.

End SemStart.

Section SemStop.

(*Stop pre-condition*)
Definition pre_stop (ic: iCmp)(s: State) : Prop :=
    exists c:Cmp, cmpInstalled c s /\ (running s) ic c.

(*Stop post-condition*)
Definition post_stop (ic:iCmp)(s s':State) : Prop :=
     (*stopIns: Remove the running instance*)
     (forall (ic':iCmp)(c':Cmp), (running s') ic' c' -> (running s) ic' c') /\
     (forall (ic':iCmp)(c':Cmp), (running s) ic' c' -> (running s') ic' c' \/ ic=ic') /\
     insNotInState ic s' /\
     (*revokeTPermsIns: If any temporal permission was delegated to the instance, revoke it.*)
     (forall (ic':iCmp)(cp:CProvider)(u:uri)(pt:PType), (delTPerms s') ic' cp u pt -> 
       (delTPerms s) ic' cp u pt) /\
     (forall (ic':iCmp)(cp:CProvider)(u:uri)(pt:PType), (delTPerms s) ic' cp u pt -> 
       (delTPerms s') ic' cp u pt \/ ic' = ic) /\
     (forall (cp:CProvider)(u:uri)(pt:PType), ~ (delTPerms s') ic cp u pt) /\
     
     apps s = apps s' /\ perms s = perms s' /\ defPerms s = defPerms s'/\
       delPPerms s = delPPerms s' /\ resCont s = resCont s' /\ systemImage s = systemImage s'.

End SemStop.

Section SemRead.

(*Component c has the permissions necessary to read resource u from
  content provider cp*)
Definition canRead (c:Cmp)(cp:CProvider)(s:State) : Prop :=
 forall (a1 a2:App)(Hc:inApp c a1)(Hcp: inApp (cmpCP cp) a2), 
  (apps s) a1 /\ (apps s) a2 ->
   let m2 := manifest a2 in
    ((exp m2) (cmpCP cp) Hcp /\
    let readEn := (readE m2) cp Hcp in
    let cmpEn := (cmpE m2) (cmpCP cp) Hcp in
    let appEn := appE m2 in
    forall p:Perm,  (readEn = Some p) \/
              (readEn = None /\ cmpEn = Some p) \/
               (readEn = None /\ cmpEn = None /\ appEn = Some p)
                -> (perms s) a1 p) \/
   a1 = a2.

(*Component c has delegated permissions to perform pt on resource u
  from content provider cp*)
Definition delPerms (c:Cmp)(cp:CProvider)(u:uri)(pt:PType)(s:State) :=
 forall (a:App), 
  apps s a ->
   inApp c a ->
   (exists ic':iCmp, exists c':Cmp, (running s) ic' c' /\ inApp c' a /\ (delTPerms s) ic' cp u Both) \/
   (exists ic':iCmp, exists c':Cmp, (running s) ic' c' /\ inApp c' a /\ (delTPerms s) ic' cp u pt) \/
   (delPPerms s) a cp u Both \/
   (delPPerms s) a cp u pt.

(*Read pre-condition*)
Definition pre_read (ic:iCmp)(cp:CProvider)(u:uri)(s:State) : Prop :=
     cmpInstalled (cmpCP cp) s /\ existsRes cp u s /\
     (exists c:Cmp, cmpInstalled c s /\ (running s) ic c /\ ~isCProvider c /\ 
       (canRead c cp s \/ delPerms c cp u Read s)).


(*Read post-condition*)
Definition post_read (ic:iCmp)(cp:CProvider)(u:uri)(s s':State) : Prop :=
     s = s'.

End SemRead.

Section SemWrite.

(*A component has necessary permissions to rewrite a resource from content provider cp*)
Definition canWrite (c:Cmp)(cp:CProvider)(s:State) : Prop :=
 forall (a1 a2:App)(Hc:inApp c a1)(Hcp: inApp (cmpCP cp) a2), 
  (apps s) a1 /\ (apps s) a2 ->
   let m2 := manifest a2 in
    ((exp m2) (cmpCP cp) Hcp /\
    let writeEn := (writeE m2) cp Hcp in
    let cmpEn := (cmpE m2) (cmpCP cp) Hcp in
    let appEn := appE m2 in
    forall p:Perm, (writeEn = Some p) \/
              (writeEn = None /\ cmpEn = Some p) \/
               (writeEn = None /\ cmpEn = None /\ appEn = Some p)
                -> perms s a1 p) \/
    a1 = a2.

(*Write pre-condition*)
Definition pre_write (ic:iCmp)(cp:CProvider)(u:uri)(newV:Val)(s:State) : Prop :=
     cmpInstalled (cmpCP cp) s /\ existsRes cp u s /\
     (exists c:Cmp, cmpInstalled c s /\ (running s) ic c /\ ~isCProvider c /\ 
        (canWrite c cp s \/ delPerms c cp u Write s)).

(*Write post-condition*)
Definition post_write (ic:iCmp)(cp:CProvider)(u:uri)(val:Val)(s s':State) : Prop :=
       (*Update the value of the written resource*)
       (forall (a':App)(r:res)(v:Val), (resCont s) a' r v -> (resCont s') a' r v \/ (inApp (cmpCP cp) a' /\ apps s a' /\
                                                               map_apply uri_eq (map_res cp) u = Value uri r)) /\
       (forall (a':App)(r:res)(v:Val), (resCont s') a' r v -> (resCont s) a' r v \/ (inApp (cmpCP cp) a' /\ apps s a' /\
							        map_apply uri_eq (map_res cp) u = Value uri r /\
               						        v = val)) /\
       (forall (a':App)(r:res), inApp (cmpCP cp) a' -> apps s a' -> map_apply uri_eq (map_res cp) u = Value uri r -> 
         (resCont s') a' r val) /\
       (forall (a':App)(r:res)(v:Val), inApp (cmpCP cp) a' -> apps s a' -> map_apply uri_eq (map_res cp) u = Value uri r -> 
         resCont s a' r v -> ~(resCont s') a' r v \/ v = val) /\

       apps s = apps s'  /\ perms s = perms s' /\ defPerms s = defPerms s' /\ running s = running s' /\
        delPPerms s = delPPerms s' /\ delTPerms s = delTPerms s' /\ systemImage s = systemImage s'.

End SemWrite.

Section SemGrantT.

(*Content provider cp allows permission delegation on resource u*)
Definition canGrant (cp:CProvider)(u:uri)(s:State) : Prop :=
     forall (a:App)(H: inApp (cmpCP cp) a), 
      (apps s) a ->
       let m := manifest a in
        (uriP m) cp H u \/
         ~(exists u':uri, (uriP m) cp H u') /\ (grantU m) cp H.

(*GrantT pre-condition*)
Definition pre_grantT (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType)(s:State) : Prop :=
     cmpInstalled (cmpCP cp) s /\ canGrant cp u s /\ existsRes cp u s /\ 
     cmpInstalled (cmpAct act) s /\
     exists c:Cmp, cmpInstalled c s /\ (running s) ic c /\
      canStart c (cmpAct act) s /\
      match pt with
      | Read => canRead c cp s \/ delPerms c cp u Read s (*Redelegation is allowed*)
      | Write => canWrite c cp s \/ delPerms c cp u Write s
      | Both => (canRead c cp s \/ delPerms c cp u Read s) /\ (canWrite c cp s \/ delPerms c cp u Write s)
      end.

(*GrantT post-condition*)
Definition post_grantT (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType)(s s':State) : Prop :=
     let cAct := cmpAct act in
      (*Create a new instance in the system*)
      ( exists iact:iCmp, insNotInState iact s /\
        (*runCmp: start the instance of activity act*)
        (forall (ic':iCmp)(c':Cmp), (running s) ic' c' -> (running s') ic' c') /\ 
        (forall (ic':iCmp)(c':Cmp), (running s') ic' c' -> (running s) ic' c' \/ (iact=ic'/\cAct=c')) /\
        (running s') iact cAct /\
	(*grantTPerm: A temporal permission corresponding to the instance of act is delegated*)
        (forall (ic':iCmp)(cp':CProvider)(u':uri)(pt':PType),
         (delTPerms s) ic' cp' u' pt' -> (delTPerms s') ic' cp' u' pt') /\
        (forall (ic':iCmp)(cp':CProvider)(u':uri)(pt':PType),
         (delTPerms s') ic' cp' u' pt' -> 
          (delTPerms s) ic' cp' u' pt' \/ (iact=ic'/\cp=cp'/\u=u'/\pt=pt')) /\
        (delTPerms s') iact cp u pt ) /\
     
     apps s = apps s' /\ perms s = perms s' /\ defPerms s = defPerms s'/\
       delPPerms s = delPPerms s' /\ resCont s = resCont s' /\ systemImage s = systemImage s'.

End SemGrantT.

Section SemGrantP.

(*GrantP pre-condition*)
Definition pre_grantP (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType)(s:State) : Prop :=
     cmpInstalled (cmpCP cp) s /\ canGrant cp u s /\ existsRes cp u s /\ 
     (apps s) a /\
     exists c:Cmp, cmpInstalled c s /\ (running s) ic c /\
      match pt with
      | Read => canRead c cp s \/ delPerms c cp u Read s (*Redelegation is allowed*)
      | Write => canWrite c cp s \/ delPerms c cp u Write s (*Permanent permissions can be delegated, as long as we have temporal permissions *)
      | Both => (canRead c cp s \/ delPerms c cp u Read s) /\ (canWrite c cp s \/ delPerms c cp u Write s)
      end.

Definition post_grantP (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType)(s s':State) : Prop :=
       (*grantPPerm: The permission is permanently granted to a*)
       (forall (a':App)(cp':CProvider)(u':uri)(pt':PType),
         (delPPerms s) a' cp' u' pt' -> (delPPerms s') a' cp' u' pt') /\
       (forall (a':App)(cp':CProvider)(u':uri)(pt':PType),
         (delPPerms s') a' cp' u' pt' -> 
           (delPPerms s) a' cp' u' pt' \/ (a=a'/\cp=cp'/\u=u'/\pt=pt')) /\
        (delPPerms s') a cp u pt /\

        apps s = apps s' /\ perms s = perms s' /\ defPerms s = defPerms s' /\ 
         running s = running s' /\ delTPerms s = delTPerms s' /\ resCont s = resCont s' /\
          systemImage s = systemImage s'.

End SemGrantP.

Section SemRevoke.

(*Revoke pre-condition*)
Definition pre_revoke (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType)(s:State) : Prop :=
      cmpInstalled (cmpCP cp) s /\ existsRes cp u s /\
      exists c:Cmp, cmpInstalled c s /\ (running s) ic c /\
        (*c only needs permissions to access resource u in cp*)
        match pt with
        | Read => canRead c cp s (*Delegated permissions cannot be revoked *)
        | Write => canWrite c cp s
        | Both => canRead c cp s /\ canWrite c cp s
        end.

(*Revoke post-condition*)
Definition post_revoke (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType)(s s':State) : Prop :=
      (*revokeTPerm: Remove temporal permissions for u in content
        provider cp to do the operation pt*)
      (forall (ic':iCmp)(cp':CProvider)(u':uri)(pt':PType),
        (delTPerms s') ic' cp' u' pt' -> (delTPerms s) ic' cp' u' pt') /\
      (forall (ic':iCmp)(cp':CProvider)(u':uri)(pt':PType),
        (delTPerms s) ic' cp' u' pt' -> (delTPerms s') ic' cp' u' pt' \/
                                       pt = Both /\ cp'=cp /\ u'=u \/
                                        pt <> Both /\ cp'=cp /\ u'=u /\ pt'=pt) /\
      (pt = Both -> forall (ic':iCmp)(pt':PType), ~ (delTPerms s') ic' cp u pt') /\
      (pt <> Both -> forall ic':iCmp, ~ (delTPerms s') ic' cp u pt) /\
      (*revokePPerm: Remove permanent permissions for u in content
        provider cp to do the operation pt*)
      (forall (a':App)(cp':CProvider)(u':uri)(pt':PType),
        (delPPerms s') a' cp' u' pt' -> (delPPerms s) a' cp' u' pt') /\
      (forall (a':App)(cp':CProvider)(u':uri)(pt':PType),
        (delPPerms s) a' cp' u' pt' -> (delPPerms s') a' cp' u' pt' \/
                                       pt = Both /\ cp'=cp /\ u'=u \/
                                       pt <> Both /\ cp'=cp /\ u'=u /\ pt'=pt) /\
      (pt = Both -> forall (a':App)(pt':PType), ~ (delPPerms s') a' cp u pt') /\
      (pt <> Both -> forall a':App, ~ (delPPerms s') a' cp u pt) /\

      apps s = apps s' /\ perms s = perms s' /\ defPerms s = defPerms s' /\
       running s = running s' /\ resCont s = resCont s' /\ systemImage s = systemImage s'.

End SemRevoke.

Section SemCall.

(*Predicate to check what permissions are required by a given system call*)
Parameter permSAC : forall p:Perm, isSystemPerm p -> SACall -> Prop.

(*Call pre-condition*)
Definition pre_call (ic:iCmp)(sac:SACall)(s:State) : Prop :=
      exists c:Cmp, (running s) ic c /\
      forall (a:App)(p:Perm)(H:isSystemPerm p), (apps s) a ->
        inApp c a -> permSAC p H sac -> (perms s) a p.

(*Call post-condition*)
Definition post_call (ic:iCmp)(sac:SACall)(s s':State) : Prop :=
      s = s'.

End SemCall.