(*
Filename: State.v

Representation of Android system state, including properties that must
hold to be considered valid.

*)
Require Export BasicDefs.
Section State.

(*State of the system*)
Record State := st { apps : App -> Prop;
	             perms : App -> Perm -> Prop;
                     defPerms : App -> Perm -> Prop;
                     running : iCmp -> Cmp -> Prop;
                     delPPerms : App -> CProvider -> uri -> PType -> Prop;
                     delTPerms : iCmp -> CProvider -> uri -> PType -> Prop;
                     resCont : App -> res -> Val -> Prop;
                     systemImage : App -> Prop}.

End State.

Section ValidState.
(*Auxiliary predicates and functions*)

(*Predicate to check if a component belongs to an application*)
Definition inApp (c:Cmp)(a:App) : Prop := (cmp (manifest a)) c.

(*Predicate to check if the component is a content provider*)
Definition isCProvider (c : Cmp) : Prop :=
match c with
| cmpCP _ => True
| _ => False
end.

(*Predicate to check if c belongs to an installed application*)
Definition cmpInstalled (c:Cmp) (s:State) : Prop :=
    exists a:App, (apps s) a /\ inApp c a.

(*Predicate to check if there is a resource in s pointed by URI u in
  content provider cp*)
Definition existsRes (cp : CProvider)(u:uri)(s:State) : Prop :=
	forall (a:App)(Ha:apps s a)(Hcp:inApp (cmpCP cp) a),
	 exists r:res, map_apply uri_eq (map_res cp) u = Value uri r /\
	   exists v:Val, (resCont s) a r v.

(*Returns the id of a component*)
Definition getCmpId (c:Cmp) : idCmp :=
   match c with
   | cmpAct a => idA a
   | cmpSrv s => idS s
   | cmpBR br => idB br
   | cmpCP cp => idC cp
   end.

(*Valid state conditions*)

Variable s:State.

(*Applications have unique identifiers*)
Definition allAppDifferent : Prop := 
   forall (a1 a2:App), (apps s) a1 
                         -> (apps s) a2
                          -> idAp a1 = idAp a2 -> a1 = a2.


(*Components belonging to the same application have unique identifiers*)
Definition allCmpDifferent : Prop :=
   forall (c1 c2:Cmp)(a1 a2:App), inApp c1 a1 
                                   -> inApp c2 a2 
                                    -> (apps s) a1 
                                     -> (apps s) a2
                                      -> getCmpId c1 = getCmpId c2 -> c1 = c2.

(*Every component belongs to only one application*)
Definition notRepeatedCmps : Prop :=
   forall (c:Cmp)(a1 a2:App), (apps s) a1 ->
                               (apps s) a2 ->
                                 inApp c a1 ->
                                  inApp c a2 ->
                                   a1 = a2.

(*If a component is running, it cannot be a content provider*)
Definition notCPrunning : Prop :=
   forall (ic:iCmp)(c:Cmp), (running s) ic c ->  ~isCProvider c.

(*All user defined permissions are declared in an installed application*)
Definition usrPermsDefined : Prop :=
   forall (a:App)(p:Perm), 
    (defPerms s) a p ->
     (apps s) a /\ (usrP (manifest a)) p.

(*If an application has delegated permissions on a content provider, 
then both are installed*)
Definition existsAppnCPinDel : Prop :=
   forall (a:App)(cp:CProvider)(u:uri)(pt:PType),
    (delPPerms s) a cp u pt
     -> (apps s) a /\ cmpInstalled (cmpCP cp) s /\ existsRes cp u s.

(*If there is a temporary permission delegation taking place, the
  recipient is running*)
Definition delTmpRun : Prop :=
   forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType),
    (delTPerms s) ic cp u pt ->
      cmpInstalled (cmpCP cp) s /\
        exists c:Cmp, exists a:App, inApp c a /\ (apps s) a /\ (running s) ic c.

(*If an instance is running, its corresponding component is installed*)
Definition cmpRunAppIns : Prop :=
   forall (ic:iCmp)(c:Cmp), (running s) ic c -> cmpInstalled c s.

(*A running instance can belong to only one component*)
Definition notRepeatedIns  : Prop :=
   forall (ic:iCmp)(c1:Cmp)(c2:Cmp),
    (running s) ic c1 ->
      (running s) ic c2 ->
        c1 = c2.

(*All the resources in the system are owned by only one application*)
Definition resContAppInst : Prop := 
   forall (a:App)(r:res)(v:Val),
    resCont s a r v -> apps s a.

(*All the resources in the system have only one value at any given
  time*)
Definition resContOneVal : Prop := 
   forall (a:App)(r:res)(v1 v2:Val),
    (resCont s) a r v1 -> (resCont s) a r v2 -> v1 = v2.

(*VALID STATE*)

Definition validState  : Prop :=
   allAppDifferent /\
    allCmpDifferent /\
     notRepeatedCmps /\
      notCPrunning /\
       usrPermsDefined /\
        existsAppnCPinDel /\ 
           delTmpRun /\
            cmpRunAppIns /\
             notRepeatedIns /\
              resContAppInst /\
                resContOneVal.

End ValidState.