(*
Filename: canGrant.v

The first three lemmas of this file prove that if an executing
instance could delegate permissions to read and/or write a given
resource, the instance can itself do the operation. 

The last lemma proves the possibility of temporal and permanent
permission delegation of any resource from a content provider of the
same application.

*)
Require Import Exec.

Require Import Tactics.

(*PROPERTY: If ic delegated permissions, either temporal or permanent,
to read a resource of cp pointed by u, then ic can read the
resource. *)
Lemma grantCanRead : forall (ic:iCmp)(c:Cmp)(cp:CProvider)(act:Activity)(a:App)(u:uri)(s s':State),
                           running s ic c ->
                            exec s (grantT ic cp act u Read) s' \/
                              exec s (grantP ic cp a u Read) s' ->
                                canRead c cp s \/ delPerms c cp u Read s.
Proof.
intros ic c cp act a u s s' H_run_ic_c H_dis_exec.
elim H_dis_exec; clear H_dis_exec; intros H_exec; inversion H_exec as [H_valid H_pre H_post]; 
 destruct_valid_state H_valid; unfold pre in H_exec; 
 [unfold pre_grantT in H_exec | unfold pre_grantP in H_exec];
 [destruct_pre_grantT H_pre | destruct_pre_grantP H_pre];
 [elim H_exists_c_canStart; clear H_exists_c_canStart |
  elim H_exists_c; clear H_exists_c];
 intros cc; intros H_conj;
 elim H_conj; clear H_conj; intros H_cmpInstalled_cc_s H_conj;
 elim H_conj; clear H_conj; intros H_run_ic_cc H_conj.

elim H_conj; clear H_conj; intros H_canStart H_canReadOrDel.
assert (c = cc) as H_ccc.
 unfold notRepeatedIns in H_notRepeatedIns.
 apply (H_notRepeatedIns ic c cc); assumption.
rewrite H_ccc; assumption.

assert (c = cc) as H_ccc.
 unfold notRepeatedIns in H_notRepeatedIns.
 apply (H_notRepeatedIns ic c cc); assumption.
rewrite H_ccc; assumption.
Qed.

(*PROPERTY: If ic delegated permissions, either temporal or permanent,
to write a resource of cp pointed by u, then ic can write the
resource. *)
Lemma grantCanWrite : forall (ic:iCmp)(c:Cmp)(cp:CProvider)(act:Activity)(a:App)(u:uri)(s s':State),
                           running s ic c ->
                            exec s (grantT ic cp act u Write) s' \/
                              exec s (grantP ic cp a u Write) s' ->
                                canWrite c cp s \/ delPerms c cp u Write s.
Proof.
intros ic c cp act a u s s' H_run_ic_c H_dis_exec.
elim H_dis_exec; clear H_dis_exec; intros H_exec; inversion H_exec as [H_valid H_pre H_post]; 
 destruct_valid_state H_valid; unfold pre in H_exec; 
 [unfold pre_grantT in H_exec | unfold pre_grantP in H_exec];
 [destruct_pre_grantT H_pre | destruct_pre_grantP H_pre];
 [elim H_exists_c_canStart; clear H_exists_c_canStart |
  elim H_exists_c; clear H_exists_c];
 intros cc; intros H_conj;
 elim H_conj; clear H_conj; intros H_cmpInstalled_cc_s H_conj;
 elim H_conj; clear H_conj; intros H_run_ic_cc H_conj.

elim H_conj; clear H_conj; intros H_canStart H_canReadOrDel.
assert (c = cc) as H_ccc.
 unfold notRepeatedIns in H_notRepeatedIns.
 apply (H_notRepeatedIns ic c cc); assumption.
rewrite H_ccc; assumption.

assert (c = cc) as H_ccc.
 unfold notRepeatedIns in H_notRepeatedIns.
 apply (H_notRepeatedIns ic c cc); assumption.
rewrite H_ccc; assumption.
Qed.

(*PROPERTY: If ic delegated permissions, either temporal or permanent,
to read and write a resource of cp pointed by u, then ic can read and write the
resource. *)
Lemma grantCanRW : forall (ic:iCmp)(c:Cmp)(cp:CProvider)(act:Activity)(a:App)(u:uri)(s s':State),
                           running s ic c ->
                            exec s (grantT ic cp act u Both) s' \/
                             exec s (grantP ic cp a u Both) s' ->
                              (canRead c cp s \/ delPerms c cp u Read s) /\ 
                                (canWrite c cp s \/ delPerms c cp u Write s).
Proof.
intros ic c cp act a u s s' H_run_ic_c H_dis_exec.
elim H_dis_exec; clear H_dis_exec; intros H_exec; inversion H_exec as [H_valid H_pre H_post]; 
 destruct_valid_state H_valid; unfold pre in H_exec; 
 [unfold pre_grantT in H_exec | unfold pre_grantP in H_exec];
 [destruct_pre_grantT H_pre | destruct_pre_grantP H_pre];
 [elim H_exists_c_canStart; clear H_exists_c_canStart |
  elim H_exists_c; clear H_exists_c];
 intros cc; intros H_conj;
 elim H_conj; clear H_conj; intros H_cmpInstalled_cc_s H_conj;
 elim H_conj; clear H_conj; intros H_run_ic_cc H_conj.

elim H_conj; clear H_conj; intros H_canStart H_canReadOrDel.
assert (c = cc) as H_ccc.
 unfold notRepeatedIns in H_notRepeatedIns.
 apply (H_notRepeatedIns ic c cc); assumption.
rewrite H_ccc; assumption.

assert (c = cc) as H_ccc.
 unfold notRepeatedIns in H_notRepeatedIns.
 apply (H_notRepeatedIns ic c cc); assumption.
rewrite H_ccc; assumption.
Qed.

(*PROPERTY: If c and cp belong to the same application, and cp
authorizes the delegation of permissions over u, then ic can delegate
those permissions, either temporarily or permanently. *)
Lemma grantSameApp : forall (a:App)(ic:iCmp)(c:Cmp)(cp:CProvider)(act:Activity)(a':App)(u:uri)(pt:PType)(s:State),
                       validState s ->
                        apps s a -> 
                         running s ic c ->
                          inApp c a ->
                           inApp (cmpCP cp) a ->
                            cmpInstalled (cmpAct act) s ->
                             apps s a' ->
                              canStart c (cmpAct act) s ->
                               canGrant cp u s -> 
                                existsRes cp u s -> 
                                 pre_grantT ic cp act u pt s /\
                                   pre_grantP ic cp a' u pt s.
Proof.
intros a ic c cp act a' u pt s H_valid H_appa H_run_ic_c H_inApp_c_a H_inApp_cp_a H_cmpInstalled_act_s H_appa' H_canStart_c_act H_canGrant_cp H_existsRes_u.
destruct_valid_state H_valid.
split.

(*grantT*)
unfold pre_grantT.
split.
unfold cmpInstalled.
exists a; split; assumption.
split.
assumption.
split.
assumption.
split.
assumption.
exists c.
split.
unfold cmpInstalled.
exists a; split; assumption.
split.
assumption.
split.
assumption.
case pt; simpl.
left.
unfold canRead.
intros a1 a2 H_c_a1 H_cp_a2 H_conj.
right.
elim H_conj. clear H_conj. intros H_appa1 H_appa2.
unfold notRepeatedCmps in H_notRepeatedCmps.
assert (a1 = a) as H_a1a.
 apply (H_notRepeatedCmps c a1 a); assumption.
assert (a2 = a) as H_a2a.
 apply (H_notRepeatedCmps (cmpCP cp) a2 a); assumption.
rewrite H_a1a. rewrite H_a2a.
reflexivity.

left.
unfold canWrite.
intros a1 a2 H_inApp_a1 H_inApp_cp_a2 H_conj.
right.
elim H_conj. clear H_conj. intros H_appa1 H_appa2.
unfold notRepeatedCmps in H_notRepeatedCmps.
assert (a1 = a) as H_a1a.
 apply (H_notRepeatedCmps c a1 a); assumption.
assert (a2 = a) as H_a2a.
 apply (H_notRepeatedCmps (cmpCP cp) a2 a); assumption.
rewrite H_a1a. rewrite H_a2a.
reflexivity.

split;
left;
[unfold canRead | unfold canWrite];
intros a1 a2 H_inApp_c_a1 H_inApp_cp_a2 H_conj;
right;
elim H_conj; clear H_conj; intros H_appa1 H_appa2;
unfold notRepeatedCmps in H_notRepeatedCmps.
assert (a1 = a) as H_a1a.
 apply (H_notRepeatedCmps c a1 a); assumption.
assert (a2 = a) as H_a2a.
 apply (H_notRepeatedCmps (cmpCP cp) a2 a); assumption.
rewrite H_a1a. rewrite H_a2a.
reflexivity.

assert (a1 = a) as H_a1a.
 apply (H_notRepeatedCmps c a1 a); assumption.
assert (a2 = a) as H_a2a.
 apply (H_notRepeatedCmps (cmpCP cp) a2 a); assumption.
rewrite H_a1a. rewrite H_a2a.
reflexivity.

(*grantP*)
unfold pre_grantP.
split.
unfold cmpInstalled.
exists a; split; assumption.
split.
assumption.
split.
assumption.
split.
assumption.
exists c.
split.
unfold cmpInstalled.
exists a; split; assumption.
split.
assumption.
case pt; simpl.
left.
unfold canRead.
intros a1 a2 H_inApp_c_a1 H_inApp_cp_a2 H_conj.
right.
elim H_conj. clear H_conj. intros H_appa1 H_appa2.
unfold notRepeatedCmps in H_notRepeatedCmps.
assert (a1 = a) as H_a1a.
 apply (H_notRepeatedCmps c a1 a); assumption.
assert (a2 = a) as H_a2a.
 apply (H_notRepeatedCmps (cmpCP cp) a2 a); assumption.
rewrite H_a1a. rewrite H_a2a.
reflexivity.

left.
unfold canWrite.
intros a1 a2 H_inApp_c_a1 H_inApp_cp_a2 H_conj.
right.
elim H_conj. clear H_conj. intros H_appa1 H_appa2.
unfold notRepeatedCmps in H_notRepeatedCmps.
assert (a1 = a) as H_a1a.
 apply (H_notRepeatedCmps c a1 a); assumption.
assert (a2 = a) as H_a2a.
 apply (H_notRepeatedCmps (cmpCP cp) a2 a); assumption.
rewrite H_a1a. rewrite H_a2a.
reflexivity.

split;
left;
[unfold canRead | unfold canWrite];
intros a1 a2 H_inApp_c_a1 H_inApp_cp_a2 H_conj;
right;
elim H_conj; clear H_conj; intros H_appa1 H_appa2;
unfold notRepeatedCmps in H_notRepeatedCmps.
assert (a1 = a) as H_a1a.
 apply (H_notRepeatedCmps c a1 a); assumption.
assert (a2 = a) as H_a2a.
 apply (H_notRepeatedCmps (cmpCP cp) a2 a); assumption.
rewrite H_a1a. rewrite H_a2a.
reflexivity.

assert (a1 = a) as H_a1a.
 apply (H_notRepeatedCmps c a1 a); assumption.
assert (a2 = a) as H_a2a.
 apply (H_notRepeatedCmps (cmpCP cp) a2 a); assumption.
rewrite H_a1a. rewrite H_a2a.
reflexivity.
Qed.