(* 
Filename: canReadWrite.v

The first two lemmas of this file give necessary conditions for the
read and write of content providers. Then, we prove that a component
can perform both operations on any content provider of the same
application. Finally, we give a necessary and sufficient condition for
reading and writing non-exported content providers. *)
Require Import Exec.

Require Import Tactics.

(*PROPERTY: If ic could read the resource in cp at u, then its
associated component belongs to an application that has the
permissions for reading that component (either since its installation,
or by a delegation). *)
Lemma icCanReadcp : forall (ic:iCmp)(c:Cmp)(cp:CProvider)(u:uri)(s s':State),
                           running s ic c ->
                            exec s (read ic cp u) s' ->
                             canRead c cp s \/ delPerms c cp u Read s.
Proof.
intros ic c cp u s s' H_run_ic_c H_exec.
inversion H_exec as [H_valid H_pre H_post].
destruct_valid_state H_valid.
destruct_pre_read H_pre.
elim H_exists_c_canRead. clear H_exists_c_canRead.
intros cc. intros H_conj.
elim H_conj. clear H_conj. intros H_cmpInstalled_cc_s H_conj.
elim H_conj. clear H_conj. intros H_run_ic'_cc H_conj.
elim H_conj. clear H_conj. intros H_notCP_cc H_canRead_cc_c_s.
unfold notRepeatedIns in H_notRepeatedIns.
assert (cc = c) as H_ccc.
 apply (H_notRepeatedIns ic cc c); assumption.
rewrite <- H_ccc.
assumption.
Qed.


(*PROPERTY: If ic could write the resource in cp at u, then its
associated component belongs to an application that has the
permissions for writing that component (either since its installation,
or by a delegation). *)
Lemma icCanWritecp : forall (ic:iCmp)(c:Cmp)(cp:CProvider)(u:uri)(v:Val)(s s':State),
                           running s ic c ->
                            exec s (write ic cp u v) s' ->
                             canWrite c cp s \/ delPerms c cp u Write s.
Proof.
intros ic c cp u v s s' H_run_ic_c H_exec.
inversion H_exec as [H_valid H_pre H_post].
destruct_valid_state H_valid.
destruct_pre_write H_pre.
elim H_exists_c_canWrite. clear H_exists_c_canWrite.
intros cc. intros H_conj.
elim H_conj. clear H_conj. intros H_cmpInstalled_cc_s H_conj.
elim H_conj. clear H_conj. intros H_run_ic'_cc H_conj.
elim H_conj. clear H_conj. intros H_notCP_cc H_canWrite_cc_c_s.
unfold notRepeatedIns in H_notRepeatedIns.
assert (cc = c) as H_ccc.
 apply (H_notRepeatedIns ic cc c); assumption.
rewrite <- H_ccc.
assumption.
Qed.

(*PROPERTY: If cp belongs to the same application that c, then c can
read cp *)
Lemma readSameApp : forall (ic:iCmp)(c:Cmp)(cp:CProvider)(u:uri)(a:App)(s:State),
                          validState s ->
                           running s ic c ->
                            apps s a ->
                             inApp c a ->
                              inApp (cmpCP cp) a ->
                               existsRes cp u s ->
                                pre_read ic cp u s.
Proof.
intros ic c cp u a s H_valid H_run_ic_c H_appa H_inApp_c_a H_inApp_cp_a H_existsRes_u_s.
destruct_valid_state H_valid.
unfold pre_read.
split.
unfold cmpInstalled.
exists a.
split; assumption.
split.
assumption.
exists c.
split.
unfold cmpInstalled.
exists a.
split; assumption.
split.
assumption.
split.
unfold notCPrunning in H_notCPrunning.
apply (H_notCPrunning ic c).
assumption.
left.
unfold canRead.
intros a1 a2 H_inApp_c_a1 H_inApp_cp_a2 H_conj.
right.
elim H_conj. clear H_conj. intros H_appa1 H_appa2.
unfold notRepeatedCmps in H_notRepeatedCmps.
assert (a1 = a) as H_a1a.
 apply (H_notRepeatedCmps c a1 a); assumption.
assert (a2 = a) as H_a2a.
 apply (H_notRepeatedCmps (cmpCP cp) a2 a); assumption.
rewrite H_a1a. rewrite H_a2a.
reflexivity.
Qed.

(*PROPERTY: If cp belongs to the same application that c, then c can
write cp *)
Lemma writeSameApp : forall (ic:iCmp)(c:Cmp)(cp:CProvider)(u:uri)(v:Val)(a:App)(s:State),
                          validState s ->
                           running s ic c ->
                            apps s a ->
                             inApp c a ->
                              inApp (cmpCP cp) a ->
                               existsRes cp u s ->
                                pre_write ic cp u v s.
Proof.
intros ic c cp u v a s H_valid H_run_ic_c H_appa H_inApp_c_a H_inApp_cp_a H_existsRes_u_s.
destruct_valid_state H_valid.
unfold pre_write.
split.
unfold cmpInstalled.
exists a.
split; assumption.
split.
assumption.
exists c.
split.
unfold cmpInstalled.
exists a.
split; assumption.
split.
assumption.
split.
unfold notCPrunning in H_notCPrunning.
apply (H_notCPrunning ic c).
assumption.
left.
unfold canRead.
intros a1 a2 H_inApp_c_a1 H_inApp_cp_a2 H_conj.
right.
elim H_conj. clear H_conj. intros H_appa1 H_appa2.
unfold notRepeatedCmps in H_notRepeatedCmps.
assert (a1 = a) as H_a1a.
 apply (H_notRepeatedCmps c a1 a); assumption.
assert (a2 = a) as H_a2a.
 apply (H_notRepeatedCmps (cmpCP cp) a2 a); assumption.
rewrite H_a1a. rewrite H_a2a.
reflexivity.
Qed.

(*PROPERTY: If cp is not exported, and c belongs to other application,
cp can be read by c iff its application has delegated permissions for
that. *)
Lemma readNotExp : forall (a a':App)(ic:iCmp)(c:Cmp)(cp:CProvider)(H:inApp (cmpCP cp) a')(u:uri)(s:State),
                          validState s ->
                           existsRes cp u s -> 
                            apps s a /\ apps s a' ->
                             running s ic c ->
                              inApp c a ->
                               ~ exp (manifest a') (cmpCP cp) H ->
                                a <> a' ->
                                 iff (pre_read ic cp u s) (delPerms c cp u Read s).
Proof.
intros a a' ic c cp H_inApp_cp_a' u s H_valid H_existsRes_u_s H_appa1Anda2 H_run_ic_c H_inApp_c_a H_notExp_cp H_aNeqa'.
destruct_valid_state H_valid.
unfold iff.
split.
(*->*)
intros H_pre.
destruct_pre_read H_pre.
elim H_exists_c_canRead. clear H_exists_c_canRead.
intros c1. intros H_conj.
elim H_conj. clear H_conj. intros H_cmpInstalled_c1_s H_conj.
elim H_conj. clear H_conj. intros H_run_ic'_c1 H_conj.
elim H_conj. clear H_conj. intros H_notCP_c1 H_canRead_c1_c_s.
assert (c = c1) as H_cc1.
  unfold notRepeatedIns in H_notRepeatedIns.
  apply (H_notRepeatedIns ic c c1); assumption.
rewrite <- H_cc1 in H_canRead_c1_c_s.
elim H_canRead_c1_c_s; clear H_canRead_c1_c_s.
 intros H_canRead.
 unfold canRead in H_canRead.
 elim (H_canRead a a' H_inApp_c_a H_inApp_cp_a' H_appa1Anda2). clear H_canRead. intros H_conj.
 elim H_conj; clear H_conj; intros H_exp H_perms.
  absurd (exp (manifest a') (cmpCP cp) H_inApp_cp_a'); assumption.

  intros H_aa'.
  absurd (a = a'); assumption.

trivial.
(*<-*)
intros H_delPerms.
elim H_appa1Anda2. clear H_appa1Anda2. intros H_appa1 H_appa2.
unfold pre_read.
split.
unfold cmpInstalled.
exists a'; split; assumption.
split.
assumption.
exists c.
split.
unfold cmpInstalled.
exists a.
split; assumption.
split.
assumption.
split.
unfold notCPrunning in H_notCPrunning.
apply (H_notCPrunning ic c); assumption.
right.
assumption.
Qed.

(*PROPERTY: If cp is a non-exported content provider, it can be
written by an external component iff that component has delegated
permissions to do it. *)
Lemma writeNotExp : forall (a a':App)(ic:iCmp)(c:Cmp)(cp:CProvider)(H:inApp (cmpCP cp) a')(u:uri)(v:Val)(s:State),
                          validState s ->
                           existsRes cp u s -> 
                            apps s a /\ apps s a' ->
                             running s ic c ->
                              inApp c a ->
                               ~ exp (manifest a') (cmpCP cp) H ->
                                a <> a' ->
                                 iff (pre_write ic cp u v s) (delPerms c cp u Write s).
Proof.
intros a a' ic c cp H_inApp_cp_a' u v s H_valid H_existsRes_u_s H_appa1Anda2 H_run_ic_c H_inApp_c_a H_notExp_cp H_aNeqa'.
destruct_valid_state H_valid.
unfold iff.
split.
(*->*)
intros H_pre.
destruct_pre_write H_pre.
elim H_exists_c_canWrite. clear H_exists_c_canWrite.
intros c1. intros H_conj.
elim H_conj. clear H_conj. intros H_cmpInstalled_c1_s H_conj.
elim H_conj. clear H_conj. intros H_run_ic'_c1 H_conj.
elim H_conj. clear H_conj. intros H_notCP_c1 H_canWrite_c1_c_s.
assert (c = c1) as H_cc1.
  unfold notRepeatedIns in H_notRepeatedIns.
  apply (H_notRepeatedIns ic c c1); assumption.
rewrite <- H_cc1 in H_canWrite_c1_c_s.
elim H_canWrite_c1_c_s; clear H_canWrite_c1_c_s.
 intros H_canWrite.
 unfold canWrite in H_canWrite.
 elim (H_canWrite a a' H_inApp_c_a H_inApp_cp_a' H_appa1Anda2). clear H_canWrite. intros H_conj.
 elim H_conj; clear H_conj; intros H_exp H_perms.
  absurd (exp (manifest a') (cmpCP cp) H_inApp_cp_a'); assumption.

  intros H_aa'.
  absurd (a = a'); assumption.

trivial.
(*<-*)
intros H_delPerms.
elim H_appa1Anda2. clear H_appa1Anda2. intros H_appa1 H_appa2.
unfold pre_write.
split.
unfold cmpInstalled.
exists a'; split; assumption.
split.
assumption.
exists c.
split.
unfold cmpInstalled.
exists a.
split; assumption.
split.
assumption.
split.
unfold notCPrunning in H_notCPrunning.
apply (H_notCPrunning ic c); assumption.
right.
assumption.
Qed.