(*
Filename: cmpRunAppIns.v

We prove that the state resulting from the execution of any action
preserves the property cmpRunAppIns. For that, an auxiliary lemma is
proven for each action.

*)
Require Import Tactics.

Require Export Exec.

Section AuxLemmas.

Variables s s' : State.

Lemma proyCmpRunAppIns : validState s -> cmpRunAppIns s.
Proof.
intros H_valid.
destruct_valid_state H_valid.
assumption.
Qed.

Lemma installPresCmpRunAppIns : forall (a:App), exec s (install a) s' -> 
                                                    cmpRunAppIns s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyCmpRunAppIns H_valid); intros H_cmpRunAppIns.
unfold cmpRunAppIns in *.
intros ic c H_run_ic_c_s'.
(*ic is an executing instance of component c in s'. We have to show
that c is installed in s'. *)
destruct_pre_install H_pre.
destruct_post_install H_post.
(*Component c was already installed in s*)
assert (cmpInstalled c s) as H_cmpInstalled.
 apply (H_cmpRunAppIns ic c).
 (*If ic is running in s', it was running in s*)
 rewrite H_sim_1.
 assumption.
unfold cmpInstalled in H_cmpInstalled.
elim H_cmpInstalled. clear H_cmpInstalled.
intros aa. intros H_conj.
(*aa is the application owner of c, installed in s*)
elim H_conj. clear H_conj. intros H_appaa H_inApp_c_aa.
unfold cmpInstalled.
(*We use aa as witness for the property in s'*)
exists aa.
split.
apply H_addApp_1; assumption.
assumption.
Qed.

Lemma uninstallPresCmpRunAppIns : forall (a:App), exec s (uninstall a) s' ->
                                                    cmpRunAppIns s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyCmpRunAppIns H_valid); intros H_cmpRunAppIns.
unfold cmpRunAppIns in *.
intros ic c H_run_ic_c_s'.
destruct_pre_uninstall H_pre.
destruct_post_uninstall H_post.
(*Component c was already installed in s*)
assert (cmpInstalled c s) as H_cmpInstalled_c_s.
 apply (H_cmpRunAppIns ic c).
 (*If ic is running in s', it was running in s*)
 rewrite H_sim_1.
 assumption.
unfold cmpInstalled in H_cmpInstalled_c_s.
elim H_cmpInstalled_c_s. clear H_cmpInstalled_c_s.
intros aa. intros H_conj.
(*aa is the application owner of c, installed in s*)
elim H_conj. clear H_conj. intros H_appaa H_inApp_c_aa.
unfold cmpInstalled.
(*We use aa as witness for the property in s'*)
exists aa.
(* Either aa is still installed in s', or aa was uninstalled with the execution of the action. *)
elim (H_removeApp_2 aa H_appaa).
 (*Case 1: aa is still installed in s'*)
 intros H_appaa_s'.
 split; assumption.

 (*Case 2: aa was uninstalled. Absurd.*)
 intros H_aaa.
 (* To uninstall aa, none of its components can be running in s, but
 ic is running*)
 absurd (inApp c aa).
  rewrite H_aaa.
  apply (H_not_running ic c).
  rewrite H_sim_1; assumption.

  assumption.
Qed. 
 
Lemma startPresCmpRunAppIns : forall (ic:iCmp)(c:Cmp), exec s (start ic c) s' ->
                                                         cmpRunAppIns s'.
Proof.
intros ic c H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyCmpRunAppIns H_valid); intros H_cmpRunAppIns.
unfold cmpRunAppIns in *.
intros ic' c' H_run_ic_c_s'.
destruct_pre_start H_pre.
destruct_post_start H_post.
elim H_runCmp. clear H_runCmp. intros icc H_conj.
elim H_conj. clear H_conj. intros H_insNotInState_icc_s H_conj.
(*icc is the new instance that is added  to the system*)
elim H_conj. clear H_conj. intros H_runCmp_1 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_2 H_runCmp_3.
(* Either c' is the component c of the start action, or it is a different component*)
assert (running s ic' c' \/ icc = ic' /\ c = c') as H_dis.
 apply H_runCmp_2; assumption.
elim H_dis; clear H_dis.
(*Case 1: instance ic' of component c' was already running in s*)
 intros H_run_ic'_c'_s.
 (* By hypothesis, we know that c' is installed in s*)
 assert (cmpInstalled c' s) as H_cmpInstalled_c'_s.
  apply (H_cmpRunAppIns ic'); assumption.
(*Since installed apps do not change after the start action, if c'
  is installed in s, it must also be installed in s'*)
 unfold cmpInstalled.
 unfold cmpInstalled in H_cmpInstalled_c'_s.
 rewrite H_sim_1 in H_cmpInstalled_c'_s.
 elim H_cmpInstalled_c'_s.
 intros aa. intros H_conj.
 exists aa; assumption.

(*Case 2: Component c' is the one starting, with instance icc*)
 intros H_conj.
 elim H_conj. clear H_conj. intros H_iccic' H_cc'.
 (*We use the start action precondition, since c=c'*)
 rewrite H_cc' in H_cmpInstalled_c_s.
 unfold cmpInstalled in H_cmpInstalled_c_s.
 elim H_cmpInstalled_c_s. clear H_cmpInstalled_c_s.
 intros aa. intros H_conj.
 unfold cmpInstalled.
 rewrite H_sim_1 in H_conj.
 exists aa; assumption.
Qed.

Lemma stopPresCmpRunAppIns : forall (ic:iCmp), exec s (stop ic) s' ->
                                                 cmpRunAppIns s'.
Proof.
intros ic H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyCmpRunAppIns H_valid); intros H_cmpRunAppIns.
unfold cmpRunAppIns in *.
intros ic' c H_run_ic_c_s'.
destruct_pre_stop H_pre.
destruct_post_stop H_post.
assert (running s ic' c) as H_run_ic'_c.
 apply H_stopIns_1; assumption.
assert (cmpInstalled c s) as H_cmpInstalled_c_s.
 apply (H_cmpRunAppIns ic' c); assumption.
unfold cmpInstalled in *.
rewrite <- H_sim_1.
elim H_cmpInstalled_c_s.
intros aa. intros H_conj.
exists aa; assumption.
Qed.

Lemma callPresCmpRunAppIns : forall (ic:iCmp)(sac:SACall), exec s (call ic sac) s' ->
                                                             cmpRunAppIns s'.
Proof.
intros ic sac H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyCmpRunAppIns H_valid); intros H_cmpRunAppIns.
unfold cmpRunAppIns in *.
intros ic' c H_run_ic_c_s'.
unfold post in H_post.
unfold post_call in H_post.
rewrite <- H_post.
apply (H_cmpRunAppIns ic' c).
rewrite H_post.
assumption.
Qed.

Lemma readPresCmpRunAppIns : forall (ic:iCmp)(cp:CProvider)(u:uri), exec s (read ic cp u) s' ->
                                                                      cmpRunAppIns s'.
Proof.
intros ic cp u H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyCmpRunAppIns H_valid); intros H_cmpRunAppIns.
unfold cmpRunAppIns in *.
intros ic' c H_run_ic_c_s'.
rewrite <- H_post.
apply (H_cmpRunAppIns ic' c).
rewrite H_post.
assumption.
Qed.

Lemma writePresCmpRunAppIns : forall (ic:iCmp)(cp:CProvider)(u:uri)(v:Val), exec s (write ic cp u v) s' ->
                                                                                 cmpRunAppIns s'.
Proof.
intros ic cp u v H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyCmpRunAppIns H_valid); intros H_cmpRunAppIns.
unfold cmpRunAppIns in *.
intros ic' c H_run_ic_c_s'.
destruct_post_write H_post.
unfold cmpInstalled.
rewrite <- H_sim_1.
apply (H_cmpRunAppIns ic' c).
rewrite H_sim_4.
assumption.
Qed.

Lemma grantTPresCmpRunAppIns : forall (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType), exec s (grantT ic cp act u pt) s' ->
                                                                                                 cmpRunAppIns s'.
Proof.
intros ic cp act u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyCmpRunAppIns H_valid); intros H_cmpRunAppIns.
unfold cmpRunAppIns in *.
intros ic' c H_run_ic_c_s'.
(*ic' is a running instance of c' in s'. Then c' is installed*)
destruct_pre_grantT H_pre.
destruct_post_grantT H_post.
elim H_runAndGrantT. clear H_runAndGrantT.
intros icc. intros H_conj.
elim H_conj. clear H_conj. intros H_insNotinState_icc_s H_conj.
elim H_conj. clear H_conj. intros H_runCmp_1 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_2 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_3 H_conj.
elim H_conj. clear H_conj. intros H_grantTPerm_1 H_conj.
elim H_conj. clear H_conj. intros H_grantTPerm_2 H_grantTPerm_3.
(*Either c' is the component c of the start action, or it is a different one. *)
assert (running s ic' c \/ icc = ic' /\ cmpAct act = c) as H_dis.
 apply H_runCmp_2; assumption.
elim H_dis; clear H_dis.
(*Case 1: The instance ic' of c' was already running in s*)
 intros H_run_ic'_c.
 (*By hypothesis, we know that c' was installed in s*)
 assert (cmpInstalled c s) as H_cmpInstalled_c_s.
  apply (H_cmpRunAppIns ic'); assumption.
(*Since installed apps do not change after the start action, if c'
  is installed in s, it must also be installed in s'*)
 unfold cmpInstalled.
 unfold cmpInstalled in H_cmpInstalled_c_s.
 rewrite H_sim_1 in H_cmpInstalled_c_s.
 elim H_cmpInstalled_c_s.
 intros aa. intros H_conj.
 exists aa; assumption.

(*Case 2: Component c' is the one starting, with instance icc*)
 intros H_conj.
 elim H_conj. clear H_conj. intros H_iccic' H_actc.
 rewrite H_actc in H_cmpInstalled_act_s.
 unfold cmpInstalled.
 rewrite <- H_sim_1.
 assumption.
Qed.

Lemma grantPPresCmpRunAppIns : forall (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType), exec s (grantP ic cp a u pt) s' ->
                                                                                         cmpRunAppIns s'.
Proof.
intros ic cp a u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyCmpRunAppIns H_valid); intros H_cmpRunAppIns.
unfold cmpRunAppIns in *.
intros ic' c H_run_ic_c_s'.
destruct_post_grantP H_post.
unfold cmpInstalled.
rewrite <- H_sim_1.
apply (H_cmpRunAppIns ic' c).
rewrite H_sim_4.
assumption.
Qed.

Lemma revokePresCmpRunAppIns : forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType), exec s (revoke ic cp u pt) s' ->
                                                                                  cmpRunAppIns s'.
Proof.
intros ic cp u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyCmpRunAppIns H_valid); intros H_cmpRunAppIns.
unfold cmpRunAppIns in *.
intros ic' c H_run_ic_c_s'.
destruct_post_revoke H_post.
unfold cmpInstalled.
rewrite <- H_sim_1.
apply (H_cmpRunAppIns ic' c).
rewrite H_sim_4.
assumption.
Qed.

End AuxLemmas.

Section cmpRunAppInsInv.

Theorem cmpRunAppInsInv : forall (a:Action)(s s':State), exec s a s' -> cmpRunAppIns s'.
Proof.
intros a s s'.
induction a.

exact (installPresCmpRunAppIns s s' a).
exact (uninstallPresCmpRunAppIns s s' a).
exact (startPresCmpRunAppIns s s' i c).
exact (stopPresCmpRunAppIns s s' i).
exact (callPresCmpRunAppIns s s' i s0).
exact (readPresCmpRunAppIns s s' i c u).
exact (writePresCmpRunAppIns s s' i c u v).
exact (grantTPresCmpRunAppIns s s' i c a u p).
exact (grantPPresCmpRunAppIns s s' i c  a u p).
exact (revokePresCmpRunAppIns s s' i c u p).
Qed.

End cmpRunAppInsInv.