(* 
Filename: delTmpRun.v

Proof that the state resulting from the execution of any action
preserves the property delTmpRun. For that, an auxiliary lemma is
proved for each action.

*)
Require Import Tactics.

Require Export Exec.

Section AuxLemmas.

Variables s s' : State.

Lemma proyDelTmpRun : validState s -> delTmpRun s.
Proof.
intros H_valid.
destruct_valid_state H_valid.
assumption.
Qed.

Lemma installPresDelTmpRun : forall (a:App), exec s (install a) s' -> 
                                                    delTmpRun s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyDelTmpRun H_valid); intros H_delTmpRun.
unfold delTmpRun in *.
intros ic cp u pt H_ic_delTPerms.
(*ic is a running instance of a component that has delegated
  permissions in s' to access the resource identified by u of content
  provider cp to do operation pt. We have to show that cp is installed
  in s', that ic is running and that its corresponding component is
  installed in s'. *)
destruct_pre_install H_pre.
destruct_post_install H_post.
(*Application installation does not modify temporal permissions that
  were given to ic; therefore ic already had the permissions in s. *)
assert (delTPerms s ic cp u pt).
 rewrite H_sim_3; assumption.
(* The property on ic and cp holds in s *)
assert (cmpInstalled (cmpCP cp) s /\ (exists c : Cmp, exists a : App, inApp c a /\ apps s a /\ running s ic c)) as H_conj.
 apply (H_delTmpRun ic cp u pt); assumption.
elim H_conj. clear H_conj. intros H_cp_cmpInstalled H_exists.
split.
elim H_exists. clear H_exists.
intros cc. intros H_exists.
elim H_cp_cmpInstalled. clear H_cp_cmpInstalled. intros aa H_conj.
unfold cmpInstalled.
exists aa.
elim H_conj. clear H_conj. intros H_appaa H_inApp_cc_aa.
split.
apply H_addApp_1; assumption.
assumption.
elim H_exists. clear H_exists.
intros cc. intros H_exists.
elim H_exists. clear H_exists.
intros aa. intros H_conj.
exists cc.
exists aa.
elim H_conj. clear H_conj. intros H_inApp_cc_aa H_conj.
elim H_conj. clear H_conj. intros H_appaa H_run_ic_cc.
split.
assumption.
split.
apply H_addApp_1; assumption.
rewrite H_sim_1 in H_run_ic_cc.
assumption.
Qed.

Lemma uninstallPresDelTmpRun : forall (a:App), exec s (uninstall a) s' ->
                                                 delTmpRun s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyDelTmpRun H_valid); intros H_delTmpRun.
unfold delTmpRun in *.
intros ic cp u pt H_ic_delTPerms.
(*ic is a running instance of a component that has delegated
  permissions in s' to access the resource identified by u of content
  provider cp to do operation pt. We have to show that cp is installed
  in s', that ic is running and that its corresponding component is
  installed in s'. *)
destruct_pre_uninstall H_pre.
destruct_post_uninstall H_post.

(*If ic has delegated permissions in s', it already had them in s,
  before uninstalling a. *)
assert (delTPerms s ic cp u pt) as H_delTPerms_ic_u_s.
 apply H_revokeOtherTPerms_1; assumption.
(* The property on ic and cp holds in s *)
assert (cmpInstalled (cmpCP cp) s /\ (exists c : Cmp, exists a : App, inApp c a /\ apps s a /\ running s ic c)) as H_conj.
 apply (H_delTmpRun ic cp u pt); assumption.
elim H_conj. clear H_conj. intros H_cmpInstalled_cp H_exists. 
split.
elim H_cmpInstalled_cp. clear H_cmpInstalled_cp.
intros aa. intros H_conj.
(*aa is the owner of cp in s*)
unfold cmpInstalled.
(*We have to show that aa is still the owner of cp in s' *)
exists aa.
elim H_conj. clear H_conj. intros H_appaa H_inApp_cp_aa.
split.

(*Either aa is still installed in s', or aa is the uninstalled application *)
elim (H_removeApp_2 aa H_appaa).
 (*Case 1: aa is installed in s'*)
 trivial.
 (*Case 2: aa is the uninstalled application (this is absurd) *)
 intros H_aaa.
 rewrite <- H_aaa in H_revokeOtherTPerms_3.
 (*By hypothesis, after uninstall, no application can have delegated
   permissions on a content provider owned by aa. But ic has
   permissions on cp (owned by aa). Abusrd *)
 absurd (delTPerms s' ic cp u pt).
  apply H_revokeOtherTPerms_3; assumption.
  assumption.
assumption.

elim H_exists. clear H_exists.
intros cc H_exists.

(*cc is the component corresponding to ic *)
elim H_exists. clear H_exists.
intros aa. intros H_conj.

(*aa is the application that owns cc *)
elim H_conj. clear H_conj. intros H_inApp_cc_aa H_conj.
elim H_conj. clear H_conj. intros H_appaa H_run_ic_cc_s.

(*We have to show that cc is sitll the component corresponding to ic
  in s', and that aa is the application that owns cc (which is
  installed) *)
exists cc. exists aa.
split.
assumption.
split.

(*Either aa is installed in s', or aa is the uninstalled application*)
elim (H_removeApp_2 aa H_appaa).
 (*Case 1: aa is installed in s'*)
 trivial.
 (*Case 2: aa was uninstalled. This is absurd. *)
 intros H_aaa.
 rewrite <- H_aaa in H_not_running.

 (*If aa was uninstalled, the uninstall pre-condition must hold in s.
   In particular, if an instance is running in s, it cannot belong to
   any component owned by aa. Since ic is running in s, and it is an
   instance of cc, cc cannot be owneed by aa. *)
 assert (~ inApp cc aa).
  apply (H_not_running ic cc); assumption.
 absurd (inApp cc aa); assumption.
rewrite H_sim_1 in H_run_ic_cc_s.
assumption.
Qed.

Lemma startPresDelTmpRun : forall (ic:iCmp)(c:Cmp), exec s (start ic c) s' ->
                                         delTmpRun s'.
Proof.
intros ic c H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyDelTmpRun H_valid); intros H_delTmpRun.
unfold delTmpRun in *.
intros ic' cp u pt H_ic_delTPerms.
(*ic' is a running instance of a component that has delegated
  permissions in s' to access the resource identified by u of content
  provider cp to do operation pt. We have to show that cp is installed
  in s', that ic' is running and that its corresponding component is
  installed in s'. *)
destruct_pre_start H_pre.
destruct_post_start H_post.
(*If ic' has delegated permissions in s', it already had them in s *)
rewrite <- H_sim_5 in H_ic_delTPerms.
(*Therefore, the property holds on s *)
assert (cmpInstalled (cmpCP cp) s /\ 
          exists c : Cmp, exists a : App, inApp c a /\ apps s a /\ running s ic' c) as H_conj.
 apply (H_delTmpRun ic' cp u pt); assumption.
elim H_conj. clear H_conj. intros H_cmpInstalled_cp_s H_exists.
split.
unfold cmpInstalled. unfold cmpInstalled in H_cmpInstalled_cp_s.
rewrite H_sim_1 in H_cmpInstalled_cp_s.
assumption.
elim H_exists. clear H_exists.
intros cc. intros H_exists.
(*cc is the component corresponding to instance ic' in s *)
elim H_exists. clear H_exists.
intros aa. intros H_conj.
(*aa is the application that owns cc in s *)
elim H_conj. clear H_conj. intros H_inApp_cc_aa H_conj.
elim H_conj. clear H_conj. intros H_appaa H_run_ic'_cc_s.
(*We will use aa and cc as witness to prove the property on s' *)
exists cc. exists aa.
split.
assumption.
split.
rewrite H_sim_1 in H_appaa.
assumption.
elim H_runCmp. clear H_runCmp. intros icc H_conj.
elim H_conj. clear H_conj. intros H_insNotInState_icc_s H_conj.
elim H_conj. clear H_conj. intros H_runCmp_1 H_conj.
apply H_runCmp_1.
assumption.
Qed.

Lemma stopPresDelTmpRun : forall (ic:iCmp), exec s (stop ic) s' ->
                                             delTmpRun s'.
Proof.
intros ic H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyDelTmpRun H_valid); intros H_delTmpRun.
unfold delTmpRun in *.
intros ic' cp u pt H_ic_delTPerms.
(*ic' is a running instance of a component that has delegated
  permissions in s' to access the resource identified by u of content
  provider cp to do operation pt. We have to show that cp is installed
  in s', that ic' is running and that its corresponding component is
  installed in s'. *)
destruct_pre_stop H_pre.
destruct_post_stop H_post.
(*If ic' has delegated permissions in s', it already had them in s*)
assert (delTPerms s ic' cp u pt) as H_delTPerms_ic'_u.
 apply H_revokeTPermsIns_1; assumption.
(*Therefore, the property holds in s *)
assert (cmpInstalled (cmpCP cp) s /\
         (exists c : Cmp, exists a : App, inApp c a /\ apps s a /\ running s ic' c)) as H_conj.
 apply (H_delTmpRun ic' cp u pt); assumption.
elim H_conj. clear H_conj. intros H_cmpInstalled_cp_s H_exists.
split.
unfold cmpInstalled. unfold cmpInstalled in H_cmpInstalled_cp_s.
rewrite H_sim_1 in H_cmpInstalled_cp_s.
assumption.
elim H_exists. clear H_exists.
intros cc. intros H_exists.
(*cc is the component corresponding to ic' in s *)
elim H_exists. clear H_exists.
intros aa. intros H_conj.
(*aa is the application that owns cc in s *)
elim H_conj. clear H_conj. intros H_inApp_cc_aa H_conj.
elim H_conj. clear H_conj. intros H_appaa H_run_ic'_cc.
(*We will use aa and cc as witness to prove the property on s' *)
exists cc. exists aa.
split.
assumption.
split.
rewrite H_sim_1 in H_appaa.
assumption.
(*Either ic' is running in s', or it was stopped with the execution of
  this action *)
elim (H_stopIns_2 ic' cc H_run_ic'_cc).
 (*Case 1: ic' is running in s'*)
 trivial.

 (*Case 2: ic' was stopped *)
 intros H_icic'.
 (*By hypothesis, ic' has delegated permissions in s'. However, since
   ic' was stopped, it cannot have delegated permissions in s'.
   Abusrd. *)
 absurd (delTPerms s' ic' cp u pt).
  rewrite H_icic' in H_revokeTPermsIns_3.
  apply H_revokeTPermsIns_3.
  assumption.
Qed.

Lemma callPresDelTmpRun : forall (ic:iCmp)(sac:SACall), exec s (call ic sac) s' ->
                                                          delTmpRun s'.
Proof.
intros ic sac H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyDelTmpRun H_valid); intros H_delTmpRun.
unfold delTmpRun in *.
intros ic' cp u pt H_ic_delTPerms.
unfold post in H_post.
unfold post_call in H_post.
rewrite <- H_post in H_ic_delTPerms.
assert (cmpInstalled (cmpCP cp) s /\
    (exists c : Cmp, exists a : App, inApp c a /\ apps s a /\ running s ic' c)) as H_conj.
 apply (H_delTmpRun ic' cp u pt); assumption.
unfold cmpInstalled in H_conj.
unfold existsRes in H_conj.
rewrite H_post in H_conj.
assumption.
Qed.

Lemma readPresDelTmpRun : forall (ic:iCmp)(cp:CProvider)(u:uri), exec s (read ic cp u) s' ->
                                                                   delTmpRun s'.
Proof.
intros ic cp u H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyDelTmpRun H_valid); intros H_delTmpRun.
unfold delTmpRun in *.
intros ic' cp' u' pt H_ic_delTPerms.
rewrite <- H_post in H_ic_delTPerms.
assert (cmpInstalled (cmpCP cp') s /\
    (exists c : Cmp, exists a : App, inApp c a /\ apps s a /\ running s ic' c)) as H_conj.
 apply (H_delTmpRun ic' cp' u' pt); assumption.
unfold cmpInstalled in H_conj.
unfold existsRes in H_conj.
rewrite H_post in H_conj.
assumption.
Qed.

Lemma writePresDelTmpRun : forall (ic:iCmp)(cp:CProvider)(u:uri)(v:Val), exec s (write ic cp u v) s' ->
                                                                           delTmpRun s'.
Proof.
intros ic cp u v H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyDelTmpRun H_valid); intros H_delTmpRun.
unfold delTmpRun in *.
intros ic' cp' u' pt H_ic_delTPerms.
destruct_post_write H_post.
rewrite <- H_sim_6 in H_ic_delTPerms.
assert (cmpInstalled (cmpCP cp') s /\
    (exists c : Cmp, exists a : App, inApp c a /\ apps s a /\ running s ic' c)) as H_conj.
 apply (H_delTmpRun ic' cp' u' pt); assumption.
unfold cmpInstalled in H_conj.
unfold existsRes in H_conj.
rewrite H_sim_1 in H_conj.
rewrite H_sim_4 in H_conj.
assumption.
Qed.

Lemma grantTPresDelTmpRun : forall (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType), exec s (grantT ic cp act u pt) s' ->
                                                                                              delTmpRun s'.
Proof.
intros ic cp act u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyDelTmpRun H_valid); intros H_delTmpRun.
unfold delTmpRun in *.
intros ic' cp' u' pt' H_ic_delTPerms.
destruct_pre_grantT H_pre.
destruct_post_grantT H_post.
elim H_runAndGrantT. clear H_runAndGrantT.
intros icc. intros H_conj.
elim H_conj. clear H_conj. intros H_insNotinState_icc_s H_conj.
elim H_conj. clear H_conj. intros H_runCmp_1 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_2 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_3 H_conj.
elim H_conj. clear H_conj. intros H_grantTPerm_1 H_conj.
elim H_conj. clear H_conj. intros H_grantTPerm_2 H_grantTPerm_3.
(*Either ic' alread had delegated permissions in s, or they were
  delegated by the grantT action *)
elim (H_grantTPerm_2 ic' cp' u' pt' H_ic_delTPerms).
 (*Case 1: ic' had the permissions in s*)
 intros H_delTPerms_ic'_u'_s.
 (*The property holds on  ic' and c' in s*)
 assert (cmpInstalled (cmpCP cp') s /\ (exists c : Cmp, exists a : App, inApp c a /\ apps s a /\ running s ic' c)) as H_conj.
  apply (H_delTmpRun ic' cp' u' pt'); assumption.
 elim H_conj. clear H_conj. intros H_cmpInstalled_cp'_s H_exists.
 split.
 elim H_cmpInstalled_cp'_s.
 intros aa. intros H_conj.
 (*aa is the application owner of cp' in s *)
 elim H_conj. clear H_conj. intros H_appaa H_inApp_cp'_aa.
 unfold cmpInstalled.
 (*We show that aa is the application owner of cp' in s' *)
 exists aa.
 split.
 rewrite H_sim_1 in H_appaa.
 assumption.
 assumption.
 elim H_exists. clear H_exists.
 intros cc. intros H_exists.

 (*cc is the component corresponding to ic' *)
 elim H_exists. clear H_exists.
 intros aa. intros H_conj.
 (*aa is the owner of cc*)
 elim H_conj. clear H_conj. intros H_inApp_cc_aa H_conj.
 elim H_conj. clear H_conj. intros H_appaa H_run_ic'_cc.  
 (*cc and aa are the witnesses for the property in s' *)
 exists cc. exists aa.
 split.
 assumption.
 split.
 rewrite H_sim_1 in H_appaa.
 assumption.
 apply H_runCmp_1; assumption.

 (*Case 2: ic' recieved permissions through the grantT action*)
 intros H_conj.
 elim H_conj. clear H_conj. intros H_iccic' H_conj.
 elim H_conj. clear H_conj. intros H_cpcp' H_conj.
 elim H_conj. clear H_conj. intros H_uu' H_ptpt'.
 split.
 rewrite H_cpcp' in H_cmpInstalled_cp_s.
 unfold cmpInstalled.
 rewrite <- H_sim_1.
 assumption.
 (*The component that corresponds to ic' is the activity that recieves
   the permission *)
 exists (cmpAct act).
 elim H_cmpInstalled_act_s.
 intros aa. intros H_conj.
 (*aa is the owner of act in s*)
 elim H_conj. clear H_conj. intros H_appaa H_inApp_act_aa.
 (*aa is the witnesss of the property in s' *)
 exists aa.
 split.
 assumption.
 split.
 rewrite H_sim_1 in H_appaa.
 assumption.
 rewrite <- H_iccic'.
 assumption.
Qed.

Lemma grantPPresDelTmpRun : forall (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType), exec s (grantP ic cp a u pt) s' ->
                                                                                        delTmpRun s'.
Proof.
intros ic cp a u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyDelTmpRun H_valid); intros H_delTmpRun.
unfold delTmpRun in *.
intros ic' cp' u' pt' H_ic_delTPerms.
destruct_post_grantP H_post.
rewrite <- H_sim_5 in H_ic_delTPerms.
assert (cmpInstalled (cmpCP cp') s /\
    (exists c : Cmp, exists a : App, inApp c a /\ apps s a /\ running s ic' c)) as H_conj.
 apply (H_delTmpRun ic' cp' u' pt'); assumption.
unfold cmpInstalled in H_conj.
unfold existsRes in H_conj.
rewrite H_sim_1 in H_conj.
rewrite H_sim_4 in H_conj.
assumption.
Qed.

Lemma revokePresDelTmpRun : forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType), exec s (revoke ic cp u pt) s' ->
                                                                                 delTmpRun s'.
Proof.
intros ic cp u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyDelTmpRun H_valid); intros H_delTmpRun.
unfold delTmpRun in *.
intros ic' cp' u' pt' H_ic_delTPerms.
(*ic' is an running instance of a component that has delegated
  permissions in s' to access the resource identified by u' of content
  provider cp' to do operation pt0. We have to show that cp' is installed
  in s', that ic' is running and that its corresponding component is
  installed in s'. *)
destruct_pre_revoke H_pre.
destruct_post_revoke H_post.
(*If ic' has delegated permissions in s', then it already had them in
  s, and were not revoked *)
assert (delTPerms s ic' cp' u' pt') as H_delTPerms_ic'_u'_s.
 apply H_revokeTPerm_1; assumption.
(*The property holds in s*)
assert (cmpInstalled (cmpCP cp') s /\ (exists c : Cmp, exists a : App, inApp c a /\ apps s a /\ running s ic' c)) as H_conj.
 apply (H_delTmpRun ic' cp' u' pt'); assumption.
elim H_conj. clear H_conj. intros H_cmpInstalled_cp'_s H_exists.
split.
elim H_cmpInstalled_cp'_s. clear H_cmpInstalled_cp'_s.
intros aa. intros H_conj.
(*aa is the owner of cp' in s *)
elim H_conj. clear H_conj. intros H_appaa H_inApp_cp'_aa.
unfold cmpInstalled.
(*aa is also owner of cp' in s' *)
exists aa.
split.
rewrite H_sim_1 in H_appaa.
assumption.
assumption.
elim H_exists. clear H_exists.
intros cc. intros H_exists.
(*cc is the component corresponding to ic'*)
elim H_exists. clear H_exists.
intros aa. intros H_conj.
(*aa is the owner of cc*)
elim H_conj. clear H_conj. intros H_inApp_cc_aa H_conj.
elim H_conj. clear H_conj. intros H_appaa H_run_ic'_cc.
(*cc and aa are the witnesses of the property *)
exists cc. exists aa.
split.
assumption.
split.
rewrite H_sim_1 in H_appaa.
assumption.
rewrite H_sim_4 in H_run_ic'_cc.
assumption.
Qed.

End AuxLemmas.

Section  delTmpRunInv.

Theorem delTmpRunInv : forall (a:Action)(s s':State), exec s a s' -> delTmpRun s'.
Proof.
intros a s s'.
induction a.

exact (installPresDelTmpRun s s' a).
exact (uninstallPresDelTmpRun s s' a).
exact (startPresDelTmpRun s s' i c).
exact (stopPresDelTmpRun s s' i).
exact (callPresDelTmpRun s s' i s0).
exact (readPresDelTmpRun s s' i c u).
exact (writePresDelTmpRun s s' i c u v).
exact (grantTPresDelTmpRun s s' i c a u p).
exact (grantPPresDelTmpRun s s' i c  a u p).
exact (revokePresDelTmpRun s s' i c u p).
Qed.

End delTmpRunInv.