(*
Filename: notCPrunning.v

We prove that the state resulting from the execution of any action
preserves the property notCPrunning. An auxiliary lemma is proved for
each action.

*)
Require Import Tactics.

Require Export Exec.

Section AuxLemmas.

Variables s s' : State.

Lemma proyNotCPrunning : validState s -> notCPrunning s.
Proof.
intros H_valid.
destruct_valid_state H_valid.
assumption.
Qed.

Lemma installPresNotCPrunning : forall (a:App), exec s (install a) s' -> 
                                                    notCPrunning s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotCPrunning H_valid); intros H_notCPrun.
unfold notCPrunning in *.
intros ic c H_run_s' H_isCP.
destruct_pre_install H_pre.
destruct_post_install H_post.
apply (H_notCPrun ic c).
rewrite H_sim_1.
assumption.
assumption.
Qed.

Lemma uninstallPresNotCPrunning : forall (a:App), exec s (uninstall a) s' -> 
                                                      notCPrunning s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotCPrunning H_valid); intros H_notCPrun.
unfold notCPrunning in *.
intros ic c H_run_s' H_isCP.
destruct_pre_uninstall H_pre.
destruct_post_uninstall H_post.
apply (H_notCPrun ic c).
rewrite H_sim_1.
assumption.
assumption.
Qed.

Lemma startPresNotCPrunning : forall (ic:iCmp)(c:Cmp), exec s (start ic c) s' -> 
                                                          notCPrunning s'.
Proof.
intros ic c H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotCPrunning H_valid); intros H_notCPrun.
unfold notCPrunning in *.
intros ic' c' H_run_s' H_isCP.
(*c' is a component such that it has a running instance ic' in s'. We
  have to show that c' is not a content provider *)
destruct_pre_start H_pre.
destruct_post_start H_post.
elim H_runCmp. clear H_runCmp. intros icc H_conj.
elim H_conj. clear H_conj. intros H_insNotInState_icc_s H_conj.
(*icc is the new instance added to the system*)
elim H_conj. clear H_conj. intros H_runCmp_1 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_2 H_runCmp_3.
(*Either ic' is an instanace of c' that was already running; or it is
  the new instance of c'=c *)
assert (running s ic' c' \/ icc = ic' /\ c = c') as H_dis.
 apply (H_runCmp_2 ic' c'); assumption.
elim H_dis; clear H_dis.
 (*Case 1: ic' is an instance of c' that was already running in s*)
 intros H_run_ic'_c'.
 apply (H_notCPrun ic' c'); assumption.
 (*Case 2: ic' is the new instance icc of component c = c' *)
 intros H_conj.
 elim H_conj. clear H_conj. intros H_iccic' H_cc'.
 (*Since c' = c, we use start action pre-condition on c*)
 rewrite H_cc' in H_notCP_c.
 absurd (isCProvider c'); assumption.
Qed.

Lemma stopPresNotCPrunning : forall (ic:iCmp), exec s (stop ic) s' -> 
                                                 notCPrunning s'.
Proof.
intros ic H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotCPrunning H_valid); intros H_notCPrun.
unfold notCPrunning in *.
intros ic' c H_run_s' H_isCP.
(* c is a component with instance ic', running in s'. We have to show
  that c is not a content provider. *)
destruct_pre_stop H_pre.
destruct_post_stop H_post.
(*If ic' is running in s', it was running in s*)
assert (running s ic' c) as H_run_ic'_c.
 apply H_stopIns_1; assumption.
apply (H_notCPrun ic' c).
assumption.
assumption.
Qed.

Lemma callPresNotCPrunning : forall (ic:iCmp)(sac:SACall), exec s (call ic sac) s' ->
                                                             notCPrunning s'.
Proof.
intros ic sac H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotCPrunning H_valid); intros H_notCPrun.
unfold notCPrunning in *.
intros ic' c H_run_s' H_isCP.
unfold post in H_post.
unfold post_call in H_post.
apply (H_notCPrun ic' c).
rewrite H_post.
assumption.
assumption.
Qed.

Lemma readPresNotCPrunning : forall (ic:iCmp)(cp:CProvider)(u:uri), exec s (read ic cp u) s' ->
          							      notCPrunning s'.
Proof.
intros ic cp u H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotCPrunning H_valid); intros H_notCPrun.
unfold notCPrunning in *.
intros ic' c H_run_s' H_isCP.
apply (H_notCPrun ic' c).
rewrite H_post.
assumption.
assumption.
Qed.

Lemma writePresNotCPrunning : forall (ic:iCmp)(cp:CProvider)(u:uri)(v:Val), exec s (write ic cp u v) s' ->
                                                                              notCPrunning s'.
Proof.
intros ic cp u v H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotCPrunning H_valid); intros H_notCPrun.
unfold notCPrunning in *.
intros ic' c H_run_s' H_isCP.
destruct_post_write H_post.
apply (H_notCPrun ic' c).
rewrite H_sim_4.
assumption.
assumption.
Qed.

Lemma grantTPresNotCPrunning : forall (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType), exec s (grantT ic cp act u pt) s' ->
                                                                                                 notCPrunning s'.
Proof.
intros ic cp act u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotCPrunning H_valid); intros H_notCPrun.
unfold notCPrunning in *.
intros ic' c H_run_s' H_isCP.
destruct_pre_grantT H_pre.
destruct_post_grantT H_post.
elim H_runAndGrantT. clear H_runAndGrantT.
intros icc. intros H_conj.
elim H_conj. clear H_conj. intros H_insNotinState_icc_s H_conj.
elim H_conj. clear H_conj. intros H_runCmp_1 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_2 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_3 H_conj.
elim H_conj. clear H_conj. intros H_grantTPerm_1 H_conj.
elim H_conj. clear H_conj. intros H_grantTPerm_2 H_grantTPerm_3.
(*Either ic' is an instance of c that was already running, or ic' is
  the new instance of activity act *)
assert (running s ic' c \/ icc = ic' /\ cmpAct act = c) as H_dis.
 apply (H_runCmp_2 ic' c); assumption.
elim H_dis; clear H_dis.
 (*Case 1: ic' is an instance of c that was running in s*)
 intros H_run_ic'_c.
 apply (H_notCPrun ic' c); assumption.
 (*Case 2: ic' is the new instance iff of activity act*)
 intros H_conj.
 elim H_conj. clear H_conj. intros H_iccic' H_actc.
 rewrite <- H_actc in H_isCP.
 (*If it is an activity, it is not a content provider*)
 simpl in H_isCP.
 elim H_isCP.
Qed.

Lemma grantPPresNotCPrunning : forall (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType), exec s (grantP ic cp a u pt) s' ->
                                                                                         notCPrunning s'.
Proof.
intros ic cp a u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotCPrunning H_valid); intros H_notCPrun.
unfold notCPrunning in *.
intros ic' c H_run_s' H_isCP.
destruct_post_grantP H_post.
apply (H_notCPrun ic' c).
rewrite H_sim_4.
assumption.
assumption.
Qed.

Lemma revokePresNotCPrunning : forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType), exec s (revoke ic cp u pt) s' ->
                                                                                  notCPrunning s'.
Proof.
intros ic cp u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotCPrunning H_valid); intros H_notCPrun.
unfold notCPrunning in *.
intros ic' c H_run_s' H_isCP.
destruct_post_revoke H_post.
apply (H_notCPrun ic' c).
rewrite H_sim_4.
assumption.
assumption.
Qed.

End AuxLemmas.

Section notCPrunningInv.

Theorem notCPrunningInv : forall (a:Action)(s s':State), exec s a s' -> notCPrunning s'.
Proof.
intros a s s'.
induction a.

exact (installPresNotCPrunning s s' a).
exact (uninstallPresNotCPrunning s s' a).
exact (startPresNotCPrunning s s' i c).
exact (stopPresNotCPrunning s s' i).
exact (callPresNotCPrunning s s' i s0).
exact (readPresNotCPrunning s s' i c u).
exact (writePresNotCPrunning s s' i c u v).
exact (grantTPresNotCPrunning s s' i c a u p).
exact (grantPPresNotCPrunning s s' i c  a u p).
exact (revokePresNotCPrunning s s' i c u p).
Qed.

End notCPrunningInv.